/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
Author: 1985 Thomas L. Quarles
**********/

/* load the voltage source structure with those pointers needed later 
 * for fast matrix loading 
 */

#include "spice.h"
#include <stdio.h>
#include "smpdefs.h"
#include "cktdefs.h"
#include "opampdefs.h"
#include "util.h"
#include "sperror.h"
#include "suffix.h"

/*ARGSUSED*/
int
OPAMPsetup(matrix,inModel,ckt,states)
    register SMPmatrix *matrix;
    GENmodel *inModel;
    register CKTcircuit *ckt;
    int *states;
{
    register OPAMPmodel *model = (OPAMPmodel *)inModel;
    register OPAMPinstance *here;
    int error;
    CKTnode *tmp;

    /*  loop through all the voltage source models */
    for( ; model != NULL; model = model->OPAMPnextModel ) {

        /* loop through all the instances of the model */
        for (here = model->OPAMPinstances; here != NULL ;
                here=here->OPAMPnextInstance) {
            
            if(here->OPAMPinode == 0) {
                error = CKTmkCur(ckt,&tmp,here->OPAMPname,"inode");
                if(error) return(error);
                here->OPAMPinode = tmp->number;
            }

/* macro to make elements with built in test for out of memory */
#define TSTALLOC(ptr,first,second) \
if((here->ptr = SMPmakeElt(matrix,here->first,here->second))==(double *)NULL){\
    return(E_NOMEM);\
}
            TSTALLOC(OPAMPoutplus_inode, OPAMPoutplus, OPAMPinode)
            TSTALLOC(OPAMPoutplus_outplus, OPAMPoutplus, OPAMPoutplus)
            TSTALLOC(OPAMPinplus_inplus, OPAMPinplus, OPAMPinplus)
            TSTALLOC(OPAMPinminus_inminus, OPAMPinminus, OPAMPinminus)
            TSTALLOC(OPAMPinode_inplus, OPAMPinode, OPAMPinplus)
            TSTALLOC(OPAMPinode_inminus, OPAMPinode, OPAMPinminus)
            TSTALLOC(OPAMPinode_inode, OPAMPinode, OPAMPinode)

        }
    }
    return(OK);
}

int
OPAMPunsetup(inModel,ckt)
    GENmodel *inModel;
    CKTcircuit *ckt;
{
#ifndef HAS_BATCHSIM
    OPAMPmodel *model;
    OPAMPinstance *here;

    for (model = (OPAMPmodel *)inModel; model != NULL;
	    model = model->OPAMPnextModel)
    {
        for (here = model->OPAMPinstances; here != NULL;
                here=here->OPAMPnextInstance)
	{
	    if (here->OPAMPinode) {
		CKTdltNNum(ckt, here->OPAMPinode);
		here->OPAMPinode = 0;
	    }
	}
    }
#endif
    return OK;
}
